@@ -57,17 +57,22 @@ impl Default for Beta {
 }
 
 impl Distribution for Beta {
+    type Output = f64;
     /// Samples from the given Beta distribution using the Gamma distribution.
     fn sample(&self) -> f64 {
         let x = self.alpha_gen.sample();
         x / (x + self.beta_gen.sample())
     }
+}
+
+impl Distribution1D for Beta {
     fn update(&mut self, params: &[f64]) {
         self.set_alpha(params[0]).set_beta(params[1]);
     }
 }
 
 impl Continuous for Beta {
+	type PDFType = f64;
     /// Calculates the probability density function for the given Beta function at `x`.
     ///
     /// # Remarks
@@ -81,6 +86,7 @@ impl Continuous for Beta {
 }
 
 impl Mean for Beta {
+    type MeanType = f64;
     /// Returns the mean of the beta distribution, which for a B(a, b)
     /// distribution is given by `a / (a + b)`.
     fn mean(&self) -> f64 {
@@ -89,6 +95,7 @@ impl Mean for Beta {
 }
 
 impl Variance for Beta {
+    type VarianceType = f64;
     /// Returns the variance of the beta distribution.
     fn var(&self) -> f64 {
         (self.alpha * self.beta)
@@ -105,7 +112,7 @@ mod tests {
     #[test]
     fn test_moments() {
         let dist = Beta::new(2., 4.);
-        let data = dist.sample_vec(1e6 as usize);
+        let data = dist.sample_n(1e6 as usize);
         assert_approx_eq!(dist.mean(), mean(&data), 1e-2);
         assert_approx_eq!(dist.var(), var(&data), 1e-2);
     }
