@@ -9,7 +9,7 @@ use std::ops::{Add, AddAssign, Div, DivAssign, Mul, MulAssign, Sub, SubAssign};
 
 use crate::prelude::{transpose, Dot};
 
-use super::super::utils::dot;
+use super::super::utils::{dot, ipiv_parity};
 use super::vops::*;
 use super::{broadcast_add, broadcast_div, broadcast_mul, broadcast_sub, Vector};
 
@@ -32,12 +32,12 @@ impl Matrix {
 
     /// Make a matrix filled with zeros.
     pub fn zeros(nrows: usize, ncols: usize) -> Self {
-        Self::new(Vector::zeros(nrows * ncols), nrows, ncols)
+        Self::new(Vector::zeros(nrows * ncols), nrows as i32, ncols as i32)
     }
 
     /// Make a matrix filled with ones.
     pub fn ones(nrows: usize, ncols: usize) -> Self {
-        Self::new(Vector::ones(nrows * ncols), nrows, ncols)
+        Self::new(Vector::ones(nrows * ncols), nrows as i32, ncols as i32)
     }
 
     /// Make an identity matrix with size `dims`.
@@ -54,6 +54,12 @@ impl Matrix {
         self.nrows == self.ncols
     }
 
+    /// Invert the matrix.
+    pub fn inv(&self) -> Self {
+        assert!(self.is_square(), "matrix not square");
+        self.solve(&Matrix::eye(self.nrows))
+    }
+
     /// Check whether the matrix is symmetric.
     pub fn is_symmetric(&self) -> bool {
         if self.is_square() {
@@ -77,7 +83,7 @@ impl Matrix {
         if self.shape() != other.shape() {
             return false;
         }
-        return self.data.close_to(&other.data, tol);
+        self.data.close_to(&other.data, tol)
     }
 
     /// Check whether the matrix is positive definite.
@@ -180,6 +186,18 @@ impl Matrix {
         (lu, pivots)
     }
 
+    /// Calculates the determinant of the matrix using the LU decomposition.
+    pub fn det(&self) -> f64 {
+        let (lu, p) = self.lu();
+        lu.diag().prod() * ipiv_parity(&p) as f64
+    }
+
+    /// Calculates the determinant of an LU decomposed matrix.
+    pub fn lu_det(&self, piv: &[i32]) -> f64 {
+        assert!(self.is_square(), "matrix not square");
+        self.diag().prod() * ipiv_parity(piv) as f64
+    }
+
     /// Check whether the matrix is upper triangular (i.e., all the entries below the diagonal are
     /// 0).
     pub fn is_upper_triangular(&self) -> bool {
@@ -217,17 +235,22 @@ impl Matrix {
     }
 
     /// Make a new matrix with the given number of rows and columns.
-    pub fn new<T>(data: T, nrows: usize, ncols: usize) -> Self
+    pub fn new<T>(data: T, nrows: i32, ncols: i32) -> Self
     where
         T: Into<Vector>,
     {
         let v = data.into();
+        let len = v.len();
 
-        Self {
+        let mut m = Self {
             data: v,
-            nrows,
-            ncols,
-        }
+            nrows: 1,
+            ncols: len,
+        };
+
+        m.reshape_mut(nrows, ncols);
+
+        m
     }
 
     /// Get the number of rows and columns in the matrix.
@@ -267,21 +290,22 @@ impl Matrix {
     /// Reshape the matrix. A size of `-1` in either the rows or the columns means that the size
     /// for that dimension will be automatically determined if possible.
     pub fn reshape(&self, nrows: i32, ncols: i32) -> Self {
-        let size = self.size();
-        if nrows > 0 && ncols > 0 {
+        let size = self.size() as i32;
+        let (newrows, newcols) = if nrows > 0 && ncols > 0 {
             assert_eq!(nrows * ncols, size as i32, "invalid shape");
-            Matrix::new(self.data.clone(), nrows as usize, ncols as usize)
+            (nrows, ncols)
         } else if nrows < 0 {
             assert!(nrows == -1 && ncols > 0, "invalid shape");
             // automatically determine number of rows
-            Matrix::new(self.data.clone(), size / ncols as usize, ncols as usize)
+            (size / ncols, ncols)
         } else if ncols < 0 {
             assert!(ncols == -1 && nrows > 0, "invalid shape");
             // automatically determine number of columns
-            Matrix::new(self.data.clone(), nrows as usize, size / nrows as usize)
+            (nrows, size / nrows)
         } else {
             panic!("invalid shape");
-        }
+        };
+        Matrix::new(self.data.clone(), newrows, newcols)
     }
 
     /// Apply a closure to every element in a row. The closure should take a value and return the
@@ -290,7 +314,7 @@ impl Matrix {
     where
         F: Fn(f64) -> f64,
     {
-        &mut self[row].iter_mut().for_each(|x| *x = f(*x));
+        self[row].iter_mut().for_each(|x| *x = f(*x));
     }
 
     /// Apply a closure to every element in a row.
@@ -362,7 +386,7 @@ impl Matrix {
     /// Transpose the matrix.
     pub fn t(&self) -> Self {
         let t = transpose(&self.data, self.nrows);
-        Matrix::new(t, self.ncols, self.nrows)
+        Matrix::new(t, self.ncols as i32, self.nrows as i32)
     }
 
     /// Transpose the matrix in-place.
@@ -384,8 +408,8 @@ impl Matrix {
     }
 
     /// Converts the matrix to a Vector.
-    pub fn to_vec(&self) -> Vector {
-        self.data.clone()
+    pub fn to_vec(self) -> Vector {
+        self.data
     }
 
     /// Horizontal concatenation of matrices. Adds `other` to the right of the calling matrix.
@@ -400,7 +424,11 @@ impl Matrix {
                 new_vec.push(other.data[i * other.ncols + j]);
             }
         }
-        Matrix::new(new_vec, self.nrows, self.ncols + other.ncols)
+        Matrix::new(
+            new_vec,
+            self.nrows as i32,
+            (self.ncols + other.ncols) as i32,
+        )
     }
 
     /// Vertical concatenation of matrices. Adds `other` below the calling matrix.
@@ -408,7 +436,11 @@ impl Matrix {
         assert_eq!(self.ncols, other.ncols);
         let mut new_vec = self.data.clone();
         new_vec.extend(other.data);
-        Matrix::new(new_vec, self.nrows + other.nrows, self.ncols)
+        Matrix::new(
+            new_vec,
+            (self.nrows + other.nrows) as i32,
+            self.ncols as i32,
+        )
     }
 
     /// Repeat self horizontally. That is, make `n` total copies of self and horizontally
@@ -421,14 +453,14 @@ impl Matrix {
                 new_vec.extend(&self[i]);
             }
         }
-        Matrix::new(new_vec, self.nrows, total_cols)
+        Matrix::new(new_vec, self.nrows as i32, total_cols as i32)
     }
 
     /// Repeat self vertically. That is, make `n` total copies of self and vertically
     /// concatenate them all together.
     pub fn vrepeat(&self, n: usize) -> Self {
         let total_rows = self.nrows * n;
-        Matrix::new(self.data.repeat(n), total_rows, self.ncols)
+        Matrix::new(self.data.repeat(n), total_rows as i32, self.ncols as i32)
     }
 }
 
@@ -495,17 +527,17 @@ impl Solve<Matrix> for Matrix {
             solutions.extend(self.cholesky_solve(&x));
         }
 
-        Matrix::new(solutions, system.ncols, system.nrows).t()
+        Matrix::new(solutions, system.ncols as i32, system.nrows as i32).t()
     }
 
     fn lu_solve(&self, pivots: &[i32], system: &Matrix) -> Matrix {
         let mut solutions = Vector::with_capacity(system.nrows * system.ncols);
         for i in 0..system.ncols {
             let x = system.get_col_as_vector(i);
-            solutions.extend(self.lu_solve(&pivots, &x));
+            solutions.extend(self.lu_solve(pivots, &x));
         }
 
-        Matrix::new(solutions, system.ncols, system.nrows).t()
+        Matrix::new(solutions, system.ncols as i32, system.nrows as i32).t()
     }
 
     fn solve(&self, system: &Matrix) -> Matrix {
@@ -518,7 +550,7 @@ impl Neg for Matrix {
     type Output = Self;
 
     fn neg(self) -> Self::Output {
-        Matrix::new(-self.data, self.nrows, self.ncols)
+        Matrix::new(-self.data, self.nrows as i32, self.ncols as i32)
     }
 }
 
@@ -527,7 +559,7 @@ impl PartialEq<Matrix> for Matrix {
         if self.shape() != other.shape() {
             return false;
         }
-        return self.data.eq(&other.data);
+        self.data.eq(&other.data)
     }
 }
 
@@ -607,7 +639,11 @@ macro_rules! makefn_matops {
     ($fn: ident, $innerfn: ident) => {
         pub fn $fn(m1: &Matrix, m2: &Matrix) -> Matrix {
             assert_eq!(m1.shape(), m2.shape(), "matrix shapes not equal");
-            Matrix::new($innerfn(&m1.data, &m2.data), m1.nrows, m1.ncols)
+            Matrix::new(
+                $innerfn(&m1.data, &m2.data),
+                m1.nrows as i32,
+                m1.ncols as i32,
+            )
         }
     };
 }
@@ -632,24 +668,24 @@ macro_rules! impl_mat_mat_op_helper {
 }
 
 macro_rules! impl_mat_vec_op_helper {
-    ($op: ident, $fn: ident, $innerfn: ident, $selftype: ty, $othertype: ty) => {
+    ($op: ident, $fn: ident, $innerfn: ident, $selftype: ty, $othertype: ty$(, $to_owned: ident)?) => {
         impl $op<$othertype> for $selftype {
             type Output = Matrix;
 
             fn $fn(self, other: $othertype) -> Self::Output {
-                $innerfn(&self, &other.to_matrix())
+                $innerfn(&self, &other$(.$to_owned())?.to_matrix())
             }
         }
     };
 }
 
 macro_rules! impl_vec_mat_op_helper {
-    ($op: ident, $fn: ident, $innerfn: ident, $selftype: ty, $othertype: ty) => {
+    ($op: ident, $fn: ident, $innerfn: ident, $selftype: ty, $othertype: ty$(, $to_owned: ident)?) => {
         impl $op<$othertype> for $selftype {
             type Output = Matrix;
 
             fn $fn(self, other: $othertype) -> Self::Output {
-                $innerfn(&self.to_matrix(), &other)
+                $innerfn(&self$(.$to_owned())?.to_matrix(), &other)
             }
         }
     };
@@ -668,9 +704,9 @@ macro_rules! mat_mat_op {
 macro_rules! mat_vec_op {
     ($op:ident, $fn:ident, $innerfn:ident) => {
         impl_mat_vec_op_helper!($op, $fn, $innerfn, Matrix, Vector);
-        impl_mat_vec_op_helper!($op, $fn, $innerfn, Matrix, &Vector);
+        impl_mat_vec_op_helper!($op, $fn, $innerfn, Matrix, &Vector, to_owned);
         impl_mat_vec_op_helper!($op, $fn, $innerfn, &Matrix, Vector);
-        impl_mat_vec_op_helper!($op, $fn, $innerfn, &Matrix, &Vector);
+        impl_mat_vec_op_helper!($op, $fn, $innerfn, &Matrix, &Vector, to_owned);
     };
 }
 
@@ -678,8 +714,8 @@ macro_rules! vec_mat_op {
     ($op:ident, $fn:ident, $innerfn:ident) => {
         impl_vec_mat_op_helper!($op, $fn, $innerfn, Vector, Matrix);
         impl_vec_mat_op_helper!($op, $fn, $innerfn, Vector, &Matrix);
-        impl_vec_mat_op_helper!($op, $fn, $innerfn, &Vector, Matrix);
-        impl_vec_mat_op_helper!($op, $fn, $innerfn, &Vector, &Matrix);
+        impl_vec_mat_op_helper!($op, $fn, $innerfn, &Vector, Matrix, to_owned);
+        impl_vec_mat_op_helper!($op, $fn, $innerfn, &Vector, &Matrix, to_owned);
     };
 }
 
@@ -720,14 +756,22 @@ macro_rules! impl_mat_scalar_op_for_type {
             type Output = Matrix;
 
             fn $fn(self, other: $othertype) -> Self::Output {
-                Matrix::new($fn_vs(&self.data, other), self.nrows, self.ncols)
+                Matrix::new(
+                    $fn_vs(&self.data, other),
+                    self.nrows as i32,
+                    self.ncols as i32,
+                )
             }
         }
 
         impl $op<$selftype> for $othertype {
             type Output = Matrix;
             fn $fn(self, other: $selftype) -> Self::Output {
-                Matrix::new($fn_sv(self, &other.data), other.nrows, other.ncols)
+                Matrix::new(
+                    $fn_sv(self, &other.data),
+                    other.nrows as i32,
+                    other.ncols as i32,
+                )
             }
         }
     };
@@ -784,7 +828,7 @@ macro_rules! impl_unary_ops_matrix {
                 #[doc = stringify!($op)]
                 #[doc = "` element-wise to the matrix."]
                 pub fn $op(&self) -> Self {
-                    Self::new(self.data.$op(), self.nrows, self.ncols)
+                    Self::new(self.data.$op(), self.nrows as i32, self.ncols as i32)
                 }
             }
         )+
@@ -804,7 +848,7 @@ macro_rules! impl_unaryops_with_arg_matrix {
             #[doc = stringify!($op)]
             #[doc = "` element-wise to the matrix."]
             pub fn $op(&self, arg: $argtype) -> Self {
-                Self::new(self.data.$op(arg), self.nrows, self.ncols)
+                Self::new(self.data.$op(arg), self.nrows as i32, self.ncols as i32)
             }
         }
     };
@@ -825,7 +869,7 @@ macro_rules! impl_reduction_fns_matrix {
     };
 }
 
-impl_reduction_fns_matrix!(norm, max, mean, min, std, sum, var, sample_std, sample_var);
+impl_reduction_fns_matrix!(norm, max, mean, min, std, sum, prod, var, sample_std, sample_var);
 
 #[cfg(test)]
 mod tests {
